@@ -15,16 +15,16 @@
 
 Well, at least begin testing some of the functionality
 
-$Id: testZopeGuards.py,v 1.1.4.10 2005/08/03 08:25:12 andreasjung Exp $
+$Id: testZopeGuards.py 39645 2005-10-26 17:12:35Z jim $
 """
 
-
 import os, sys
 import unittest
+from zope.testing import doctest
 import ZODB
 import AccessControl.SecurityManagement
 from AccessControl.SimpleObjectPolicies import ContainerAssertions
-from zExceptions import Unauthorized
+from AccessControl import Unauthorized
 from AccessControl.ZopeGuards \
     import guarded_getattr, get_dict_get, get_dict_pop, get_list_pop, \
     get_iter, guarded_min, guarded_max, safe_builtins, guarded_enumerate, \
@@ -78,11 +78,9 @@ class Method:
         self.args = args
 
 
-class TestGuardedGetattrPy(GuardTestCase):
+class TestGuardedGetattr(GuardTestCase):
 
     def setUp(self):
-        import AccessControl.ImplPython
-        self.guarded_getattr = AccessControl.ImplPython.guarded_getattr
         self.__sm = SecurityManager()
         self.__old = self.setSecurityManager(self.__sm)
 
@@ -100,52 +98,52 @@ class TestGuardedGetattrPy(GuardTestCase):
         self.assertEqual(rc, sys.getrefcount(value))
 
     def test_calls_validate_for_unknown_type(self):
-        self.guarded_getattr(self, 'test_calls_validate_for_unknown_type')
+        guarded_getattr(self, 'test_calls_validate_for_unknown_type')
         self.assert_(self.__sm.calls)
 
     def test_attr_handler_table(self):
-
         d = {}
         _dict = type(d)
         old = ContainerAssertions.get(_dict)
 
-        def myfactory(name, value):
-            if name == 'values':
-                return Method
-            return 1
-
-        ContainerAssertions[_dict] = myfactory
-
+        mytable = {'keys': 1,
+                   'values': Method,
+                   }
+        ContainerAssertions[_dict] = mytable
         try:
-            self.guarded_getattr(d, 'keys')
+            guarded_getattr(d, 'keys')
             self.assertEqual(len(self.__sm.calls), 0)
-            values = self.guarded_getattr(d, 'values')
+            values = guarded_getattr(d, 'values')
             self.assertEqual(values.__class__, Method)
             self.assertEqual(values.args, (d, 'values'))
+            self.assertRaises(Unauthorized, guarded_getattr, d, 'items')
         finally:
             ContainerAssertions[_dict] = old
 
-    def test_with_unicode(self):
-        # test for Collector #1617
-        d = {u'\xcf' : u'\xfc',
-             '\xab' : u'\xcd'}
-        if sys.getdefaultencoding() == 'ascii':
-            try:
-                self.guarded_getattr(d, u'\xcf')
-            except UnicodeEncodeError:
-                pass 
-        else:
-            try:
-                self.guarded_getattr(d, '\xab')
-            except AttributeError:
-                pass
-
-class TestGuardedGetattrC(TestGuardedGetattrPy):
+class TestGuardedGetitem(GuardTestCase):
 
     def setUp(self):
-        TestGuardedGetattrPy.setUp(self)
-        import AccessControl.ImplC
-        self.guarded_getattr = AccessControl.ImplC.guarded_getattr
+        self.sm = SecurityManager()
+        self.old = self.setSecurityManager(self.sm)
+
+    def tearDown(self):
+        self.setSecurityManager(self.old)
+
+    def test_guarded_getitem_passes_index_to_validate(self):
+        # BBB:  collector #1182 specifies that guarded_getitem should be
+        #       passing the 'index' to validate, rather than 'None', 
+        #       until Zope 2.9.
+        from UserDict import UserDict
+        from AccessControl.ZopeGuards import guarded_getitem
+
+        foo = []
+        protected = UserDict(foo=foo)
+
+        value = guarded_getitem(protected, 'foo')
+        self.failUnless(value is foo)
+        self.assertEqual(len(self.sm.calls), 1)
+        self.assertEqual(self.sm.calls[0],
+                         ('validate', (protected, protected, 'foo', foo)))
 
 class TestDictGuards(GuardTestCase):
 
@@ -609,6 +607,7 @@ print sneaky.private_method()
         else:
             self.fail("Didn't raise SyntaxError!")
 
+
     def test_dict_access(self):
         from RestrictedPython.tests import verify
 
@@ -634,7 +633,7 @@ print foo(**kw)
 
         self.assertEqual(its_globals['_print'](),
                         'baz\nTrue\n')
-
+        
     def _compile_str(self, text, name):
         from RestrictedPython import compile_restricted
         from AccessControl.ZopeGuards import get_safe_globals, guarded_getattr
@@ -647,6 +646,10 @@ print foo(**kw)
         g['__name__'] = __name__ # so classes can be defined in the script
         return code, g
 
+    def testPythonRealAC(self):
+        code, its_globals = self._compile("actual_python.py")
+        exec code in its_globals
+
     # Compile code in fname, as restricted Python. Return the
     # compiled code, and a safe globals dict for running it in.
     # fname is the string name of a Python file; it must be found
@@ -669,10 +672,92 @@ print foo(**kw)
             if callable(v) and v is not getattr(__builtin__, k, None):
                 d[k] = FuncWrapper(k, v)
 
+def test_inplacevar():
+    """
+Verify the correct behavior of protected_inplacevar.
+
+    >>> from AccessControl.ZopeGuards import protected_inplacevar
+
+Basic operations on objects without inplace slots work as expected:
+
+    >>> protected_inplacevar('+=', 1, 2)
+    3
+    >>> protected_inplacevar('-=', 5, 2)
+    3
+    >>> protected_inplacevar('*=', 5, 2)
+    10
+    >>> protected_inplacevar('/=', 6, 2)
+    3
+    >>> protected_inplacevar('%=', 5, 2)
+    1
+    >>> protected_inplacevar('**=', 5, 2)
+    25
+    >>> protected_inplacevar('<<=', 5, 2)
+    20
+    >>> protected_inplacevar('>>=', 5, 2)
+    1
+    >>> protected_inplacevar('&=', 5, 2)
+    0
+    >>> protected_inplacevar('^=', 7, 2)
+    5
+    >>> protected_inplacevar('|=', 5, 2)
+    7
+
+Inplace operations are allowed on lists:
+
+    >>> protected_inplacevar('+=', [1], [2])
+    [1, 2]
+
+    >>> protected_inplacevar('*=', [1], 2)
+    [1, 1]
+
+But not on custom objects:
+
+    >>> class C:
+    ...     def __iadd__(self, other):
+    ...         return 42
+    >>> protected_inplacevar('+=', C(), 2)    # doctest: +NORMALIZE_WHITESPACE
+    Traceback (most recent call last):
+    ...
+    TypeError: Augmented assignment to C objects is not allowed in
+    untrusted code
+"""
+
+if sys.version_info[:2] >= (2, 4):
+    def test_inplacevar_for_py24():
+        """
+protected_inplacevar allows inplce ops on sets:
+
+    >>> from AccessControl.ZopeGuards import protected_inplacevar
+    >>> s = set((1,2,3,4))
+    >>> sorted(protected_inplacevar('-=', s, set((1, 3))))
+    [2, 4]
+    >>> sorted(s)
+    [2, 4]
+    
+    >>> sorted(protected_inplacevar('|=', s, set((1, 3, 9))))
+    [1, 2, 3, 4, 9]
+    >>> sorted(s)
+    [1, 2, 3, 4, 9]
+
+    >>> sorted(protected_inplacevar('&=', s, set((1, 2, 3, 9))))
+    [1, 2, 3, 9]
+    >>> sorted(s)
+    [1, 2, 3, 9]
+
+    >>> sorted(protected_inplacevar('^=', s, set((1, 3, 7, 8))))
+    [2, 7, 8, 9]
+    >>> sorted(s)
+    [2, 7, 8, 9]
+
+"""
+
 def test_suite():
-    suite = unittest.TestSuite()
-    for cls in (TestGuardedGetattrPy,
-                TestGuardedGetattrC,
+    suite = unittest.TestSuite([
+        doctest.DocTestSuite(),
+        ])
+    for cls in (TestGuardedGetattr,
+                TestGuardedGetitem,
                 TestDictGuards,
                 TestBuiltinFunctionGuards,
                 TestListGuards,
