@@ -11,9 +11,9 @@
 #
 ##############################################################################
 
-__version__='$Revision: 1.90.2.12 $'[11:-2]
+__version__='$Revision: 1.96 $'[11:-2]
 
-import re, sys, os,  urllib, time, random, cgi, codecs
+import re, sys, os, time, random, codecs
 from types import StringType, UnicodeType
 from BaseRequest import BaseRequest
 from HTTPResponse import HTTPResponse
@@ -208,7 +208,6 @@ class HTTPRequest(BaseRequest):
             If the URL makes no sense in light of the current virtual
             hosting context, a ValueError is raised."""
         other = self.other
-        bad_server_url = 0
         path = filter(None, URL.split( '/'))
 
         if URL.find( '://') >= 0:
@@ -217,7 +216,6 @@ class HTTPRequest(BaseRequest):
         # Check the path against BASEPATH1
         vhbase = self._script
         vhbl = len(vhbase)
-        bad_basepath = 0
         if path[:vhbl] == vhbase:
             path = path[vhbl:]
         else:
@@ -407,7 +405,7 @@ class HTTPRequest(BaseRequest):
             CGI_name=isCGI_NAME
             defaults={}
             tainteddefaults={}
-            converter=seqf=None
+            converter=None
 
             for item in fslist:
 
@@ -456,10 +454,8 @@ class HTTPRequest(BaseRequest):
                             converter_type = type_name
                             flags=flags|CONVERTED
                         elif type_name == 'list':
-                            seqf=list
                             flags=flags|SEQUENCE
                         elif type_name == 'tuple':
-                            seqf=tuple
                             tuple_items[key]=1
                             flags=flags|SEQUENCE
                         elif (type_name == 'method' or type_name == 'action'):
@@ -1240,6 +1236,8 @@ class HTTPRequest(BaseRequest):
             if (isCGI_NAME(key) or key[:5] == 'HTTP_') and (not hide_key(key)):
                 keys[key] = 1
 
+        # Cache URLN and BASEN in self.other.
+        # This relies on a side effect of has_key.
         n=0
         while 1:
             n=n+1
@@ -1341,6 +1339,18 @@ class HTTPRequest(BaseRequest):
     def taintWrapper(self, enabled=TAINTING_ENABLED):
         return enabled and TaintRequestWrapper(self) or self
 
+    # Used by Five
+
+    def getPresentationSkin(self):
+        """see zope.component.interfaces.IPresentationRequest"""
+        return getattr(self, '_presentation_skin', None)
+
+    def setPresentationSkin(self, skin):
+        """see zope.publisher.interfaces.IPublicationRequest"""
+        self._presentation_skin = skin
+
+    def getURL(self):
+        return self.URL
 
 class TaintRequestWrapper:
     def __init__(self, req):
@@ -1415,8 +1425,7 @@ class FileUpload:
         if hasattr(file, '__methods__'): methods=file.__methods__
         else: methods= ['close', 'fileno', 'flush', 'isatty',
                         'read', 'readline', 'readlines', 'seek',
-                        'tell', 'truncate', 'write', 'writelines',
-                        '__iter__','next'] # see Collector 1837
+                        'tell', 'truncate', 'write', 'writelines']
 
         d=self.__dict__
         for m in methods:
@@ -1436,9 +1445,6 @@ class FileUpload:
         """
         return not not self.filename
 
-    def xreadlines(self):
-        return self
-
 
 parse_cookie_lock=allocate_lock()
 def parse_cookie(text,
@@ -1563,4 +1569,4 @@ def _filterPasswordFields(items):
 # if any trusted-proxies are defined in the configuration file.
 
 trusted_proxies = []
-
+    
