@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.3
+#!/home/fermigier/bin/python
 
 ##############################################################################
 #
@@ -14,7 +14,7 @@
 #
 ##############################################################################
 """
-test.py [-abcCdDfgGhLmpqrtTuv] [modfilter [testfilter]]
+test.py [-abcCdDfgGhLmprtTuv] [modfilter [testfilter]]
 
 Find and run tests written using the unittest module.
 
@@ -104,9 +104,6 @@ named .testinfo, it will not be searched for tests.  Really.)
     Keep running the selected tests in a loop.  You may experience
     memory leakage.
 
---nowarnings
-    Install a filter to suppress warnings emitted by code.
-
 -t
     Time the individual tests and print a list of the top 50, sorted from
     longest to shortest.
@@ -124,12 +121,31 @@ named .testinfo, it will not be searched for tests.  Really.)
     utility writes coverage files to a directory named `coverage' that
     is parallel to `build'.  It also prints a summary to stdout.
 
+--coverage
+    Use the coverage.py module from Gareth Rees
+    (http://www.nedbatchelder.com/code/modules/coverage.html) to collect data
+    for code coverage.  This will output trace data in a file called
+    '.coverage'. You will need to call coverage.py after the test run to
+    analyse the data (-r for a line count report, -a for annotated file).
+
+--profile
+    Use the profile module from the standard library to collect profiling data.
+    This will output data in a file called '.profile'. You will need to use the
+    pstats module from the standard library after the test run to analyse the
+    data.
+
+--hotshot
+    Use the hotshot module from the standard library to collect profiling data.
+    This will output data in a file called '.hotshot'. You will need to use the
+    hotshot.pstats module from the standard library after the test run to
+    analyse the data. You may also use the hotshot2cg script from the
+    KCacheGrind sources to create data suitable for analysis by KCacheGrind.
+
 -v
--q
     Verbose output.  With one -v, unittest prints a dot (".") for each
     test run.  With -vv, unittest prints the name of each test (for
-    some definition of "name" ...).  With no -v (or with -q), unittest is
-    silent until the end of the run, except when errors occur.
+    some definition of "name" ...).  With no -v, unittest is silent
+    until the end of the run, except when errors occur.
 
 -u
 -m
@@ -282,8 +298,7 @@ class ImmediateTestResult(unittest._TextTestResult):
         elif self.dots:
             self.stream.write(prefix[0])
             
-        if not self._progress:
-            errlist.append((test, tb))
+        errlist.append((test, tb))
 
     def startTest(self, test):
         if self._progress:
@@ -663,8 +678,8 @@ def main(module_filter, test_filter, libdir):
     if config_file:
         config_file = os.path.realpath(config_file)
         print "Parsing %s" % config_file
-        import Zope
-        Zope.configure(config_file)
+        import Zope2
+        Zope2.configure(config_file)
 
     if not keepStaleBytecode:
         from App.config import getConfiguration
@@ -744,7 +759,6 @@ def process_args(argv=None):
     global test_dir
     global config_file
     global import_testing
-    global no_warnings
 
     if argv is None:
         argv = sys.argv
@@ -755,6 +769,9 @@ def process_args(argv=None):
     LOOP = False
     GUI = False
     TRACE = False
+    COVERAGE = False
+    PROFILE = False
+    HOTSHOT = False
     REFCOUNT = False
     debug = False # Don't collect test results; simply let tests crash
     debugger = False
@@ -772,13 +789,13 @@ def process_args(argv=None):
     test_dir = None
     config_file = None
     import_testing = False
-    no_warnings = False
 
     try:
-        opts, args = getopt.getopt(argv[1:], "a:bcC:dDfg:G:hLmpqrtTuv",
+        opts, args = getopt.getopt(argv[1:], "a:bcC:dDfg:G:hLmprtTuv",
                                    ["all", "help", "libdir=", "times=",
-                                    "keepbytecode", "nowarnings", "dir=", 
-                                    "config-file=", "import-testing"])
+                                    "keepbytecode", "dir=", 
+                                    "config-file=", "import-testing",
+                                    "coverage", "profile", "hotshot"])
     except getopt.error, msg:
         print msg
         print "Try `python %s -h' for more information." % argv[0]
@@ -830,6 +847,12 @@ def process_args(argv=None):
                 print "-r ignored, because it needs a debug build of Python"
         elif k == "-T":
             TRACE = True
+        elif k == "--coverage":
+            COVERAGE = True
+        elif k == "--profile":
+            PROFILE = True
+        elif k == "--hotshot":
+            HOTSHOT = True
         elif k == "-t":
             if not timetests:
                 timetests = 50
@@ -837,8 +860,6 @@ def process_args(argv=None):
             GUI = 1
         elif k == "-v":
             VERBOSE += 1
-        elif k == "-q":
-            VERBOSE = 0
         elif k == "--times":
             try:
                 timetests = int(v)
@@ -851,12 +872,6 @@ def process_args(argv=None):
             config_file = v
         elif k == '--import-testing':
             import_testing = True
-        elif k == '--nowarnings':
-            no_warnings = True
-
-    if no_warnings:
-        import warnings
-        warnings.simplefilter('ignore', Warning, append=1)
 
     if gcthresh is not None:
         if gcthresh == 0:
@@ -918,6 +933,29 @@ def process_args(argv=None):
                           globals=globals(), locals=vars())
             r = tracer.results()
             r.write_results(show_missing=True, summary=True, coverdir=coverdir)
+
+        elif COVERAGE:
+            try:
+                from coverage import the_coverage
+            except:
+                print "You need to install coverage.py from "
+                print "http://www.nedbatchelder.com/code/modules/coverage.html"
+                sys.exit()
+            the_coverage.start()
+            main(module_filter, test_filter, libdir)
+
+        elif PROFILE:
+            import profile
+            profile.runctx("main(module_filter, test_filter, libdir)",
+                           globals=globals(), locals=vars(), 
+                           filename=".profile")
+
+        elif HOTSHOT:
+            import hotshot
+            profile = hotshot.Profile(".hotshot")
+            profile.runctx("main(module_filter, test_filter, libdir)",
+                           globals=globals(), locals=vars())
+
         else:
             bad = main(module_filter, test_filter, libdir)
             if bad:
